from . import mixins


class PaymentTypeBase(mixins.SerializableMixin):
    '''
    Base type for all payment types.  Not usable by itself.
    '''
    def serialize(self):
        return {"payment_type": self.PAYMENT_TYPE_KEY,
                self.PAYMENT_TYPE_KEY: {},
                }


class CreditCard(PaymentTypeBase):
    '''
    A payment made with a credit card.
    http://docs.veritranspay.co.id/sandbox/charge.html#vtdirect-cc
    '''
    PAYMENT_TYPE_KEY = 'credit_card'

    def __init__(self, bank, token_id, save_token_id=False):
        '''
        :param bank: Represents the acquiring bank.
        :type bank: :py:class:`str`
        :param token_id: A token retrieved from the Veritrans
            JavaScript library, after submitting the credit card details.
        :type token_id: :py:class:`str`
        :param bool save_token_id: Used in conjunction with a 2-click to
            indicate whether or not this token is to be made reusable.
        '''
        self.bank = bank
        self.token_id = token_id
        self.save_token_id = save_token_id

    def serialize(self):

        rv = super(CreditCard, self).serialize()

        rv[self.PAYMENT_TYPE_KEY] = {'bank': self.bank,
                                     'token_id': self.token_id,
                                     }

        # append save_token_id to the request, only if it's set to True
        if self.save_token_id:
            rv[self.PAYMENT_TYPE_KEY].update(
                {'save_token_id': self.save_token_id})

        return rv


class Indomaret(PaymentTypeBase):
    PAYMENT_TYPE_KEY = 'cstore'

    def __init__(self, message):
        self.store = 'Indomaret'
        self.message = message

    def serialize(self):
        rv = super(Indomaret, self).serialize()
        rv[self.PAYMENT_TYPE_KEY].update({
            'store': self.store,
            'message': self.message
        })
        return rv


class VirtualAccount(PaymentTypeBase, mixins.SerializableMixin):
    """
        Base class for payment using virtual account
    """
    PAYMENT_TYPE_KEY = 'bank_transfer'

    def serialize(self):
        rv = super(VirtualAccount, self).serialize()
        rv[self.PAYMENT_TYPE_KEY].update({
           'bank': self.bank
        })
        return rv


class VirtualAccountPermata(VirtualAccount):
    """
    A payment made with a virtual account Permata.

    https://api-docs.midtrans.com/#permata-virtual-account
    """
    def __init__(self):
        self.bank = 'permata'


class VirtualAccountBca(VirtualAccount):
    """
    A payment made with a virtual account BCA.
    NOTE: untested in sandbox, not available
    https://api-docs.midtrans.com/#bca-virtual-account
    """
    def __init__(self):
        self.bank = 'bca'


class VirtualAccountBni(VirtualAccount):
    """
    A payment made with a virtual account BNI
    NOTE: untested in sandbox, not available
    https://api-docs.midtrans.com/#bni-virtual-account
    """
    def __init__(self):
        self.bank = 'bni'


class VirtualAccountMandiri(PaymentTypeBase):
    """
     A payment made with a virtual account Mandiri (BillPayment Mandiri)

    https://api-docs.midtrans.com/#mandiri-bill-payment
    """
    PAYMENT_TYPE_KEY = 'echannel'

    def __init__(self, bill_info1, bill_info2):
        self.bill_info1 = bill_info1
        self.bill_info2 = bill_info2

    def serialize(self):
        rv = super(VirtualAccountMandiri, self).serialize()
        rv[self.PAYMENT_TYPE_KEY].update({
            'bill_info1': self.bill_info1,
            'bill_info2': self.bill_info2
        })
        return rv


class BriEpay(PaymentTypeBase):
    """
    A payment made with a Epay BRI

    https://api-docs.midtrans.com/#epay-bri
    """
    PAYMENT_TYPE_KEY = 'bri_epay'


class MandiriClickpay(PaymentTypeBase):
    """
    A payment made with a Mandiri Click Pay.

    http://api-docs.midtrans.com/#mandiri-clickpay
    """
    PAYMENT_TYPE_KEY = 'mandiri_clickpay'

    def __init__(self, card_number, input1, input2, input3, token):
        self.card_number = card_number
        self.input1 = input1
        self.input2 = input2
        self.input3 = input3
        self.token = token

    def serialize(self):
        rv = super(MandiriClickpay, self).serialize()
        rv[self.PAYMENT_TYPE_KEY].update({
            'card_number': self.card_number,
            'input1': self.input1,
            'input2': self.input2,
            'input3': self.input3,
            'token': self.token
        })

        return rv


class CimbClicks(PaymentTypeBase):
    """
    A payment made with a Cimb Clicks.

    http://api-docs.midtrans.com/#cimb-clicks
    """

    PAYMENT_TYPE_KEY = 'cimb_clicks'

    def __init__(self, description):
        self.description = description

    def serialize(self):
        rv = super(CimbClicks, self).serialize()
        rv[self.PAYMENT_TYPE_KEY].update({
            'description': self.description
        })

        return rv


class BCAKlikPay(PaymentTypeBase):
    """
    A payment made with a BCA Klik Pay.

    http://api-docs.midtrans.com/#bca-klikpay
    """
    PAYMENT_TYPE_KEY = 'bca_klikpay'

    def __init__(self, type_id, description):
        self.type = type_id
        self.description = description

    def serialize(self):
        rv = super(BCAKlikPay, self).serialize()
        rv[self.PAYMENT_TYPE_KEY].update({
            'type': self.type,
            'description': self.description
        })

        return rv


class KlikBCA(PaymentTypeBase):
    """
    A payment made with a BCA Klik Pay.

    http://api-docs.midtrans.com/#klikbca
    """
    PAYMENT_TYPE_KEY = 'bca_klikbca'

    def __init__(self, user_id, description):
        self.user_id = user_id
        self.description = description

    def serialize(self):
        rv = super(KlikBCA, self).serialize()
        rv[self.PAYMENT_TYPE_KEY].update({
            'user_id': self.user_id,
            'description': self.description})

        return rv


class GoPay(PaymentTypeBase):
    """
    A payment made with a BCA Klik Pay.

    https://api-docs.midtrans.com/#go-pay
    """
    PAYMENT_TYPE_KEY = 'gopay'


import json
import requests
from . import response


class VTDirect(object):
    '''
    Gateway used to submit requests to Veritrans via the VTDirect method.
    '''
    LIVE_API_URL = 'https://api.midtrans.com/v2'
    SANDBOX_API_URL = 'https://api.sandbox.midtrans.com/v2'

    def __init__(self, server_key, sandbox_mode=False):
        '''
        :param server_key: Your Veritrans account server key.
        :type server_key: :py:class:`str`
        :param sandbox_mode: If True, requests will be submitted to the
            Veritrans sandbox API, instead of the live API.
        :type sandbox_mode: :py:class:`bool`
        '''
        self.server_key = server_key
        self.sandbox_mode = sandbox_mode

    @property
    def base_url(self):
        '''
        Returns the Veritrans base URL for API requests.  This will
        differ depending on whether sandbox_mode is enabled or not.
        '''
        return VTDirect.SANDBOX_API_URL if self.sandbox_mode \
            else VTDirect.LIVE_API_URL

    def submit_charge_request(self, req):
        '''
        Submits a charge request to the API.  Before submitting, all the
        data in the req is validated and if a failure occurs
        a ValidationError will be raised.

        :param req: Information about a transaction and a customer to charge.
        :type req: :py:class:`veritranspay.request.ChargeRequest`
        :rtype: :py:class:`veritranspay.response.response.ChargeResponseBase`
        '''
        # run validation against our charge
        # request before submitting
        req.validate_all()

        # build up our application payload and manually
        # specify the header type.
        payload = json.dumps(req.serialize())
        headers = {'content-type': 'application/json',
                   'accept': 'application/json',
                   }

        # now cross our fingers that all went well!
        http_response = requests.post(
            '{base_url}/charge'.format(base_url=self.base_url),
            auth=(self.server_key, ''),
            headers=headers,
            data=payload)

        response_json = http_response.json()

        veritrans_response = response.build_charge_response(
            request=req,
            **response_json)

        return veritrans_response

    def submit_status_request(self, req):
        '''
        Retrieve information from Veritrans about a single transaction.

        :param req: Data about a transaction to retrieve the status of.
        :type req: :py:class:`veritranspay.request.StatusRequest` **or** Any
            response class that has an order_id attribute, such as
            :py:class:`veritranspay.response.response.ChargeResponseBase`
        :rtype: :py:class:`veritranspay.response.response.StatusResponse`
        '''
        # specifically skip if it's a response type
        # we don't have a good reason to validate those.
        if not isinstance(req, response.ResponseBase):
            req.validate_all()

        request_url_format = '{base_url}/{order_id}/status'

        headers = {'accept': 'application/json',
                   }

        http_response = requests.get(
            request_url_format.format(
                base_url=self.base_url,
                order_id=req.order_id),
            auth=(self.server_key, ''),
            headers=headers)

        response_json = http_response.json()

        veritrans_response = response.StatusResponse(**response_json)

        return veritrans_response

    def submit_cancel_request(self, req):
        '''
        Sends a request to Veritrans to cancel a single transaction.

        :param req: Data about a transaction to cancel.
        :type req: :py:class:`veritranspay.request.CancelRequest` **or** Any
            response class that has an order_id attribute, such as
            :py:class:`veritranspay.response.response.ChargeResponseBase`
        :rtype: :py:class:`veritranspay.response.response.CancelResponse`
        '''
        # specifically skip if it's a response type
        # we don't have a good reason to validate those.
        if not isinstance(req, response.ResponseBase):
            req.validate_all()

        request_url_format = '{base_url}/{order_id}/cancel'

        headers = {'accept': 'application/json',
                   }

        http_response = requests.post(
            request_url_format.format(
                base_url=self.base_url,
                order_id=req.order_id),
            auth=(self.server_key, ''),
            headers=headers)

        response_json = http_response.json()

        veritrans_response = response.CancelResponse(**response_json)

        return veritrans_response

    def submit_approval_request(self, req):
        '''
        Sends a request to Veritrans to approve a single, challenged
        transaction.

        :param req: Data about a transaction to approve.
        :type req: :py:class:`veritranspay.request.ApprovalRequest` **or** Any
            response class that has an order_id attribute, such as
            :py:class:`veritranspay.response.response.ChargeResponseBase`
        :rtype: :py:class:`veritranspay.response.response.ApproveResponse`
        '''
        # specifically skip if it's a response type
        # we don't have a good reason to validate those.
        if not isinstance(req, response.ResponseBase):
            req.validate_all()

        request_url_format = '{base_url}/{order_id}/approve'

        headers = {'accept': 'application/json',
                   }

        http_response = requests.post(
            request_url_format.format(
                base_url=self.base_url,
                order_id=req.order_id),
            auth=(self.server_key, ''),
            headers=headers)

        response_json = http_response.json()

        veritrans_response = response.ApproveResponse(**response_json)

        return veritrans_response

    def __repr__(self):
        return ("<VTDirect("
                "server_key: '{server_key}', "
                "sandbox_mode: {sandbox_mode})>"
                .format(server_key=self.server_key,
                        sandbox_mode=self.sandbox_mode))

    def register_card(self, req):
        '''
        Request a credit card to be registered to the API.  Before submitting, all the
        data in the req is validated and if a failure occurs
        a ValidationError will be raised.

        :param req: Information about a card number to be registered.
        :type req: :py:class:`veritranspay.request.RegisterCardRequest`
        :rtype: :py:class:`veritranspay.response.response.RegisterCardResponse`
        '''
        # run validation against our charge
        # request before submitting
        req.validate_all()

        # build up our application payload and manually
        # specify the header type.
        payload = req.serialize()
        headers = {}

        # now cross our fingers that all went well!
        query_params = f"card_number={payload.get('card_number')}"
        query_params += f"&card_exp_month={payload.get('card_exp_month')}"
        query_params += f"&card_exp_year={payload.get('card_exp_year')}&client_key={payload.get('client_key')}"
        if payload.get('callback') is not None:
            query_params += f"&callback={payload.get('callback')}"
        http_response = requests.get(
            f'{self.base_url}/card/register?{query_params}',

            headers=headers)

        response_json = http_response.json()

        veritrans_response = response.build_charge_response(
            request=req,
            **response_json)

        return veritrans_response



# just to make this accessible from a more sane location
from .response import *
from .response import build_charge_response, StatusResponse, ApproveResponse, \
    CancelResponse, ResponseBase, CreditCardChargeResponse, CimbsChargeResponse, \
    MandiriChargeResponse, BCAKlikPayChargeResponse, KlikBCAChargeResponse, IndomaretChargeResponse, \
    VirtualAccountBcaChargeResponse, VirtualAccountBniChargeResponse, VirtualAccountChargeResponse,\
    VirtualAccountMandiriChargeResponse, VirtualAccountPermataChargeResponse, GoPayChargeResponse, \
    RegisterCardResponse



from . import validators


class ValidatableMixin(object):
    '''
    Provides a mechanism for validating the attributes of an object instance.
    Expects a dictionary named _validators to be provided with keys that
    represent the names of attributes to be validated, mapped to an instance
    of a class in the module :py:mod:`veritranspay.validators`.
    '''
    def validate_attr(self, name, value, validator):
        '''
        Tries to validate the provided value with a given validator.
        If validation fails, a ValidationError will be raised indicating
        the name of the property the failed, along with the error message.

        :arg name: Name of the attribute being validated. This is only used
            for the error message when validation fails.
        :type name: :py:class:`str`
        :arg value: The value that will actually be validated.
        :arg validator: The appropriate validator instance that
            value will be checked with.
        :type validator: Subclass of
            :py:class:`veritranspay.validators.ValidatorBase`
        :raises: :py:class:`veritranspay.validators.ValidationError`
        '''
        try:
            validator.validate(value)
        except validators.ValidationError as e:
            msg = "{name} failed validation: {message}".format(
                name=name,
                message=e.message)
            raise validators.ValidationError(msg)

    def validate_all(self):
        '''
        Iterates over all the validators in this instances _validators
        dictionary, and validates a matching attribute on this object
        with a validator listed in the _validators dictionary.
        '''
        for attr_name in self._validators:
            value = getattr(self, attr_name)
            validator = self._validators[attr_name]
            self.validate_attr(attr_name, value, validator)


class SerializableMixin(object):
    '''
    An instance that can return a dictionary representation of it's
    properties by calling a serialize() method.
    '''
    def serialize(self):
        '''
        Returns a dictionary representing the current object.  If attributes
        of the current object also implement a serialize method, their
        dictionary representation will be added as well, instead of the object
        itself.
        :returns: Dictionary representation of an object.
        :rtype: :py:class:`dict`
        '''
        rv = {}
        for key in self.__dict__:
            val = getattr(self, key)

            # If a given attribute implements a 'serialize' method, call
            # that instead of just adding the attribute to the dictionary.
            if hasattr(val, 'serialize'):
                rv[key] = val.serialize()
            else:
                rv[key] = val

        return rv


class RequestEntity(ValidatableMixin, SerializableMixin):
    '''
    Provides no functionality, other than incorporating SerializableMixin
    and ValiditableMixin in a single base class.
    '''
    def __repr__(self):
        return '<{klass}()>'.format(klass=self.__class__.__name__)



'''
A simple set of validation classes that are used to check entities in
:py:mod:`veritranspay.request`, to give us some assurance they have
provided the required data, in the required format, for Veritrans to
accept the request.

(Although Veritrans will validate this data as well, this is done so we
don't waste time submitting data we can already know will be rejected).

When any of the Validators defined in this module fail, they raise a
:py:class:`veritranspay.validators.ValidationError`.

See the following for more details:

- http://docs.veritrans.co.id/sandbox/other_commands.html
- http://docs.veritrans.co.id/sandbox/charge.html
'''
from numbers import Number
import re

from . import constraints


class ValidationError(Exception):
    '''
    Raised whenever a validator in this module determines the value passed
    to .validate() fails validation.
    '''
    def __init__(self, message=None):
        self.message = message


class ValidatorBase(object):
    '''
    This should be the absolute base class for all validators.
    It doesn't nothing special, but allows derrived classes to call .validate()
    on super() with reckless abandon!
    '''
    def __init__(self, *args, **kwargs):
        return

    def validate(self, value):
        '''
        Given a value, raises a ValueError if validation fails, otherwise
        returns None.

        :param value: The object to test for Validation.
        '''
        return


class DummyValidator(ValidatorBase):
    '''
    This is a special case validator that never fails validation and accepts
    and parameters passed to it's constructor.
    '''
    pass


class RequiredValidator(ValidatorBase):
    '''
    Asserts that a value is not null when it's is_required attribute
    is set to 'True'
    '''
    def __init__(self, is_required=True, *args, **kwargs):
        '''
        Creates a new instance of RequiredValidator.

        :param is_required: When True (or not provided), validate()
            will fail on None values.
        :type is_required: :py:class:`bool`
        '''
        self.is_required = is_required
        super(RequiredValidator, self).__init__(**kwargs)

    def validate(self, value):
        if self.is_required and value is None:
            raise ValidationError("Required value was None")
        super(RequiredValidator, self).validate(value)


class LengthValidator(ValidatorBase):
    '''
    Asserts that a string's length is between a given
    minimum and/or maximum length.
    '''
    def __init__(self, min_length=None, max_length=None, *args, **kwargs):
        '''
        Creates a new instance of LengthValidator.

        :param min_length: Minimum required length for a string.
        :type min_length: :py:class:`int`
        :param max_length: Maximum allowed length for a string.
        :type min_length: :py:class:`int`
        '''
        if None not in [min_length, max_length]:
            if max_length < min_length:
                raise ValueError()

        self.min_length = min_length
        self.max_length = max_length

        super(LengthValidator, self).__init__(**kwargs)

    def validate(self, value):

        if self.max_length and value is not None:
            if value and len(value) > self.max_length:
                raise ValidationError(
                    "{value} longer than max_length "
                    "{max_length}".format(value=value,
                                          max_length=self.max_length))

        if self.min_length and value is not None:
            if len(value) < self.min_length:
                raise ValidationError(
                    "{value} shorter than min_length "
                    "{min_length}".format(value=value,
                                          min_length=self.min_length))

        super(LengthValidator, self).validate(value)


class RegexValidator(ValidatorBase):
    '''
    Tests a given string value against a regular expression.
    '''
    def __init__(self, pattern, *args, **kwargs):
        '''
        :param pattern: A regular expression pattern.
        :type pattern: :py:class:`str`
        '''
        self.pattern = pattern
        super(RegexValidator, self).__init__(**kwargs)

    def validate(self, value):
        # regex validator should skip its tests when value is None
        if value is not None:
            if not re.match(self.pattern, value, re.DOTALL):
                raise ValidationError(
                    "{value} did not match expected pattern"
                    "{pattern}".format(value=value,
                                       pattern=self.pattern))

        super(RegexValidator, self).validate(value)


class StringValidator(RequiredValidator, LengthValidator):
    '''
    Takes a string value.  Can optionally required the string to not be
    null by setting is_required to True (it's default),
    greater-than-or-equal to a min_length or
    less-than-or-equal-to a max_length.
    '''
    def validate(self, value):
        if value is not None:

            # python 3 removes basestring
            try:
                string_type = basestring
                # v -- this doesn't work?  works in python but not in nosetests
                # string_type = getattr(__builtins__, 'basestring', str)
            except NameError:
                string_type = str

            if not isinstance(value, string_type):
                raise ValidationError(
                    "{value} ({type}) is not "
                    "a string".format(value=value, type=type(value)))
        super(StringValidator, self).validate(value)


class NumericValidator(RequiredValidator):
    '''
    Tests that the provided value is a python numeric type.
    '''
    def validate(self, value):
        if value is not None:
            if not isinstance(value, Number):
                raise ValidationError("{value} ({type}) is not numeric".format(
                    value=value,
                    type=type(value)))
        super(NumericValidator, self).validate(value)


class AddressValidator(RequiredValidator, LengthValidator):
    '''
    Tests that a provided string is a valid length for address.
    If not required is_required should be set to false in the
    constructor.
    '''
    def __init__(self, *args, **kwargs):
        super(AddressValidator, self).__init__(
            max_length=constraints.MAX_ADDRESS_LENGTH,
            *args,
            **kwargs)


class PostalcodeValidator(RequiredValidator, RegexValidator, LengthValidator):
    '''
    Tests that a string is a valid length and format for a
    Postal Code.  It can be a maximum of 10 digits and may
    also contain spaces and hyphens (-).
    '''
    def __init__(self, *args, **kwargs):
        super(PostalcodeValidator, self).__init__(
            pattern=r'^([\d -])*$',
            max_length=constraints.MAX_POSTALCODE_LENGTH,
            *args,
            **kwargs)


class NameValidator(RequiredValidator, LengthValidator):
    '''
    Tests that a human name (eg, given or sirname) are at most 20
    characters in length.
    '''
    def __init__(self, *args, **kwargs):
        super(NameValidator, self).__init__(
            max_length=constraints.MAX_NAME_LENGTH,
            *args,
            **kwargs)


class CityValidator(NameValidator):
    '''
    Acts as an Alias for NameValidator, since their defined validation
    behavior is the same.
    '''
    pass


class PhoneValidator(RequiredValidator, RegexValidator, LengthValidator):
    '''
    Tests that a string looks like a phone number (between 5 and 19 characters)
    and only contains the characters 0-9, +, -, (, and ).
    '''
    def __init__(self, *args, **kwargs):
        super(PhoneValidator, self).__init__(
            pattern=r'^\+?[\d\-\(\) ]*$',
            min_length=constraints.MIN_PHONE_LENGTH,
            max_length=constraints.MAX_PHONE_LENGTH,
            *args,
            **kwargs)


class EmailValidator(RequiredValidator, RegexValidator, LengthValidator):
    '''
    Tests that a given string is less than 45 characters in length and
    vaguely appears to be in the proper format for an e-mail address.
    '''
    # NOTE: this is actually a REALLY complicated problem
    # http://ex-parrot.com/~pdw/Mail-RFC822-Address.html
    # the goal here is just a simple 'does it kinda look like an email'
    def __init__(self, *args, **kwargs):
        super(EmailValidator, self).__init__(
            pattern=r'^.+@.+$',
            max_length=constraints.MAX_EMAIL_LENGTH,
            *args,
            **kwargs)


class CountrycodeValidator(RequiredValidator, LengthValidator):
    '''
    Validates that a country code is in a format that Veritrans
    accepts -- which is any string less than 10 characters (note:
    their API documentation states this should be ISO 3166-1 Alpha 3)
    '''
    def __init__(self, *args, **kwargs):
        super(CountrycodeValidator, self).__init__(
            max_length=constraints.MAX_COUNTRYCODE_LENGTH,
            *args,
            **kwargs)


class PassthroughValidator(RequiredValidator):
    '''
    Allows validation of a subentity type that implements validators
    on it's own properties.  See request.ChargeRequest() for more
    information.
    If Value is an iterable, validate_all() will be called on each of
    it's elements.
    '''
    def validate(self, value):
        if value is not None:
            try:
                for child in iter(value):
                    child.validate_all()
            except TypeError:
                value.validate_all()

        super(PassthroughValidator, self).validate(value)



'''
Represents the different HTTP status codes
returned from the API to indicate errors.

http://docs.veritranspay.co.id/sandbox/status_code.html
'''
# 20x - successful submission to api
SUCCESS = 200  # NOTE: this is returned when manually cancelled too!
CHALLENGE = 201
PENDING = 201  # NOTE: returned when payment type is VirtualAccount, Cimb, Bri Epay, KlikBCA, Klikpay BCA,
EXPIRED = 202 # NOTE: returned when payment type is GoPay


# 30x
MOVED_PERMANENTLY = 300


# 40x
VALIDATION_ERROR = 400
ACCESS_DENIED = 401
# note: documented as 'Access Denied' but this is more descriptive
# and that would also create two 'Access Denied'
UNAVAILABLE_PAYMENT_TYPE = 402
DUPLICATE_ORDER_ID = 406
ACCOUNT_INACTIVE = 410
TOKEN_ERROR = 411


# 50x
SERVER_ERROR = 500
FEATURE_UNAVAILABLE = 501
BANK_CONNECTION_PROBLEM = 502
SERVER_ERROR_OTHER = 503
FRAUD_DETECTION_UNAVAILABLE = 504



from datetime import datetime


VERITRANS_DATE_TIME_FORMAT = '%Y-%m-%d %H:%M:%S'


def parse_veritrans_datetime(vt_datetime):
    '''
    Takes a string representing a date and time, in the format
    returned by Veritrans, and returns a python datetime instance.

    :param vt_datetime: String in format yyyy-mm-dd hh:mm:ss.
    :type vt_datetime: :py:class:`str`
    :rtype: :py:class:`datetime`
    '''
    return (datetime.strptime(vt_datetime, VERITRANS_DATE_TIME_FORMAT)
            if vt_datetime
            else None)


def parse_veritrans_amount(amount):
    '''
    Given an input string, returns it as an integer.  Veritrans returns
    an extra .00 appended to the end of every string, we strip that off
    because IDR is the only supported currency.

    :param amount: A string representing a numeric amount, with fractional
        currency amounts separated by '.' (NOT ,).
    :type amount: :py:class:`str`
    :rtype: :py:class:`int`
    '''
    if amount not in [None, '']:
        # only IDR supported -- so split off the fractional portion
        amt_str = amount.split('.')[0]
        return int(amt_str)
    else:
        return 0



'''
Contains entities and sub-entities that are used to create a complete
charge request to the Veritrans API.

http://docs.veritranspay.co.id/sandbox/charge.html#specification
'''
from . import mixins, validators, constraints


class Address(mixins.RequestEntity):
    '''
    Represents a physical address somewhere in the world.
    '''
    _validators = {'address': validators.AddressValidator(),
                   'city': validators.CityValidator(),
                   'postal_code': validators.PostalcodeValidator(),
                   'first_name': validators.NameValidator(is_required=False),
                   'last_name': validators.NameValidator(is_required=False),
                   'phone': validators.PhoneValidator(is_required=False),
                   'country_code': validators.CountrycodeValidator(
                       is_required=False),
                   }

    def __init__(self, address, city, postal_code,
                 first_name=None, last_name=None, phone=None,
                 country_code=None):
        '''
        :param address: Street address.
        :type address: :py:class:`str` <= 200
        :param city:  City name.
        :type city: :py:class:`str` <= 20.
        :param postal_code: Postal Code.
        :type postal_code: :py:class:`str` <= 10; numbers, hyphens '-',
            and spaces ' '
        :param first_name: Person given name.
        :type first_name: :py:class:`str` <= 20.
        :param last_name: Person surname.
        :type last_name: :py:class:`str` <= 20.
        :param phone: Phone number.
        :type phone: :py:class:`str` 5 >=< 19; numbers,
            hyphens '-', parenthesis '()',
            spaces ' '; can start with plus symbol '+'
        :param country_code: ISO-3166 alpha-3 country code.
        :type country_code: :py:class:`str` <= 10
        '''
        self.address = address
        self.city = city
        self.postal_code = postal_code
        self.first_name = first_name
        self.last_name = last_name
        self.phone = phone
        self.country_code = country_code


class TransactionDetails(mixins.RequestEntity):
    '''
    Basic information about a transaction with a customer, including
    the order id and the total amount the customer should be charged.
    '''
    _validators = {'order_id':
                   validators.StringValidator(
                       max_length=constraints.MAX_ORDERID_LENGTH),
                   'gross_amount': validators.NumericValidator(),
                   }

    def __init__(self, order_id, gross_amount):
        '''
        :param order_id: **UNIQUE** order identifier.
        :type order_id: :py:class:`str` <= 50
        :param gross_amount: Total amount the customer will be charged.
        :type gross_amount: :py:class:`int`
        '''
        self.order_id = order_id
        self.gross_amount = gross_amount


class CustomerDetails(mixins.RequestEntity):
    '''
    Personal information about a customer.
    '''
    _validators = {'first_name': validators.NameValidator(),
                   'last_name': validators.NameValidator(is_required=False),
                   'email': validators.EmailValidator(),
                   'phone': validators.PhoneValidator(),
                   'billing_address': validators.DummyValidator(),
                   'shipping_address': validators.DummyValidator(),
                   }

    def __init__(self, first_name, last_name, email, phone,
                 billing_address=None, shipping_address=None):
        '''
        :param first_name: Person given name.
        :type first_name: :py:class:`str` <= 20.
        :param last_name: Person surname.
        :type last_name: :py:class:`str` <= 20.
        :param email: Person's contact e-mail address.
        :type email: :py:class:`str` <= 45;  Must be a valid e-mail.
        :type phone: :py:class:`str` 5 >=< 19; numbers,
            hyphens '-', parenthesis '()',
            spaces ' '; can start with plus symbol '+'
        :param billing_address: Address used to validate the charge.
        :type billing_address: :py:class:`veritranspay.request.Address`
        :param shipping_address: Address where order should be shipped.
        :type shipping_address: :py:class:`veritranspay.request.Address`
        '''
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.phone = phone
        self.billing_address = billing_address
        self.shipping_address = shipping_address


class ItemDetails(mixins.RequestEntity):
    '''
    Line items details for a transaction.
    '''
    _validators = {'id': validators.StringValidator(max_length=constraints.MAX_ITEMID_LENGTH),
                   'price': validators.NumericValidator(),
                   'quantity': validators.NumericValidator(),
                   'name': validators.StringValidator(max_length=constraints.MAX_ITEMNAME_LENGTH)
                   }

    def __init__(self, item_id, price, quantity, name):
        '''
        :param item_id: Identifier for a given item.
        :type item_id: :py:class:`str` <= 50
        :param price: Unit price for a given item.
        :type price: :py:class:`int`
        :param quantity: Number of units purchased.
        :type quantity: :py:class:`int`
        :param name: Human-readable identifier for product.
        :type name: :py:class:`str` <= 50
        '''
        self.id = item_id
        self.price = price
        self.quantity = quantity
        self.name = name


class ChargeRequest(mixins.RequestEntity):
    '''
    All the information sent to Veritrans to request a customer be charged
    for a particular order.
    '''
    _validators = {'charge_type': validators.DummyValidator(),
                   'transaction_details': validators.PassthroughValidator(),
                   'customer_details': validators.PassthroughValidator(),
                   'item_details': validators.PassthroughValidator(),
                   }

    def __init__(self, charge_type, transaction_details, customer_details,
                 item_details=[]):
        '''
        :param charge_type: Account information against which to submit
            the charge.
        :type charge_type: subclass of
            :py:class:`veritranspay.payment_types.PaymentTypeBase`
        :param transaction_details: Details about the charge being submitted,
            such as the total amount to bill.
        :type transaction_details:
            :py:class:`veritranspay.request.TransactionDetails`
        :param customer_details: Personal details about the person
            being billed.
        :type customer_details:
            :py:class:`veritranspay.request.CustomerDetails`
        :param item_details: Line item details for this transaction.
        :type item_details: iterable of
            :py:class:`veritranspay.request.ItemDetails`
        '''
        self.charge_type = charge_type
        self.transaction_details = transaction_details
        self.customer_details = customer_details
        self.item_details = item_details

    def serialize(self):
        # Manually override the standard logic for serialize().  `charge_type`
        # needs to add two keys to the resulting dictionary, and all
        # other types need to be placed under specific dictionary keys.
        rv = {}
        rv.update(self.charge_type.serialize())
        rv.update({'transaction_details':
                   self.transaction_details.serialize()})
        rv.update({'customer_details': self.customer_details.serialize()})
        if self.item_details:
            rv.update({'item_details': [item.serialize() for item
                                        in self.item_details]})
        return rv


class StatusRequest(mixins.ValidatableMixin):
    '''
    Request used to retrieve information about a single charge
    that has already been submitted to Veritrans.
    '''
    _validators = {'order_id':
                   validators.StringValidator(
                       max_length=constraints.MAX_ORDERID_LENGTH),
                   }

    def __init__(self, order_id):
        '''
        :param order_id: The unique order id of the transaction.
        :type order_id: :py:class:`str` <= 50
        '''
        self.order_id = order_id


class CancelRequest(mixins.ValidatableMixin):
    '''
    Cancels a transaction.  This can only be submitted if the transaction
    is currently still pending.
    '''
    _validators = {'order_id':
                   validators.StringValidator(
                       max_length=constraints.MAX_ORDERID_LENGTH),
                   }

    def __init__(self, order_id):
        '''
        :param order_id: The unique order id of the transaction.
        :type order_id: :py:class:`str` <= 50
        '''
        self.order_id = order_id


class ApprovalRequest(mixins.ValidatableMixin):
    '''
    Approves a transaction that is currently in state 'CHALLENGE'.
    '''
    _validators = {'order_id':
                   validators.StringValidator(
                       max_length=constraints.MAX_ORDERID_LENGTH),
                   }

    def __init__(self, order_id):
        '''
        :param order_id: The unique order id of the transaction.
        :type order_id: :py:class:`str` <= 50
        '''
        self.order_id = order_id


class RegisterCardRequest(mixins.ValidatableMixin):
    '''
    All information to register card into midtrans.
    '''
    _validators = {'card_number': validators.StringValidator(),
                   'card_exp_month': validators.StringValidator(),
                   'card_exp_year': validators.StringValidator(),
                   'client_key': validators.StringValidator(),
                   'callback': validators.StringValidator(is_required=False),
                   }

    def __init__(self, card_number, card_exp_month, card_exp_year, client_key, call_back=None):
        '''
        :param card_number: Customer card number.
        :type card_number: :py:class:`str` <= 50
        :param card_exp_month: Card expired month.
        :type card_exp_month: :py:class:`int`
        :param card_exp_year: Card expired year
        :type: card_exp_year: :py:class:`int`
        :param: client_key: Client key
        :type: client_key: :py:class:`str`
        :param: callback: Url callback of site that register card.
        :type: callback: :py:class:`str`
        '''
        self.card_number = card_number
        self.card_exp_month = card_exp_month
        self.card_exp_year = card_exp_year
        self.client_key = client_key
        self.callback = call_back

    def serialize(self):
        # Manually override the standard logic for serialize().  `charge_type`
        # needs to add two keys to the resulting dictionary, and all
        # other types need to be placed under specific dictionary keys.
        rv = {
            'card_number': self.card_number,
            'card_exp_month': self.card_exp_month,
            'card_exp_year': self.card_exp_year,
            'client_key': self.client_key
        }
        if self.callback:
            rv.update({'callback': self.callback})
        return rv



'''
VeritransPay
-------

Client library for communicating with **VERSION 2** of the
veritrans.co.id payment gateway.  Note, there is no support for the older
v1 API as this was dropped in Dec 2014.

Notes
`````
This is still an early release and does not provide access to all the
functionality of the Veritrans.co.id API.  Currently, only submitting
credit-card payments, and receiving that response is supported.

Links
`````

* `veritrans website <http://veritrans.co.id/>`_

'''
from setuptools import setup


pkg_req = [
    'requests>=2.3.0',
]
test_req = pkg_req + [
    'fake-factory>=0.4.2',
    'mock>=1.0.1',
    'nose>=1.3.4',
    'coverage>=3.7.1',
]


setup(
    name='VeritransPay',
    version=__import__('veritranspay').__version__,
    url='https://github.com/derekjamescurtis/veritranspay',
    license='BSD',
    author='Derek J. Curtis',
    author_email='derek.curtis@apps-foundry.com',
    description='Veritrans.co.id API Client Library ',
    long_description=__doc__,
    packages=['veritranspay',
              'veritranspay.response',
              ],
    include_package_data=True,
    platforms='any',
    classifiers=['Development Status :: 3 - Alpha',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3.3',
                 'Programming Language :: Python :: 3.4',
                 ],
    install_requires=pkg_req,
    tests_require=test_req,
    test_suite='nose.collector'
    )



'''
These are the constraints of text field sizes provided by
veritrans.

http://docs.veritrans.co.id/sandbox/charge.html
'''

MAX_NAME_LENGTH = 20
MAX_CITY_LENGTH = 20
MAX_ADDRESS_LENGTH = 200
MAX_EMAIL_LENGTH = 45
MAX_ORDERID_LENGTH = 50
MAX_COUNTRYCODE_LENGTH = 10
MAX_ITEMNAME_LENGTH = 50
MAX_ITEMID_LENGTH = 50
MAX_POSTALCODE_LENGTH = 10

MIN_PHONE_LENGTH = 5
MAX_PHONE_LENGTH = 19



'''
Response objects are generated by a gateway after data is sent back from
Veritrans about a request.  Response formats can vary depending on the
type of error encountered from Veritrans, so the only two parameters
that are required for ALL responses are status_code, and status_message.
The rest are all purely optional (though unexpected parameters will be
ignored).

To see the expected attributes for a given type of request, check
the documentation at:

- http://docs.veritrans.co.id/sandbox/other_commands.html
- http://docs.veritrans.co.id/sandbox/charge.html
'''
from veritranspay.request import RegisterCardRequest

__all__ = ['ResponseBase', 'ChargeResponseBase', 'CreditCardChargeResponse',
           'IndomaretChargeResponse', 'CimbsChargeResponse', 'MandiriChargeResponse', 'BCAKlikPayChargeResponse',
           'KlikBCAChargeResponse', 'StatusResponse', 'CancelResponse', 'VirtualAccountChargeResponse',
           'VirtualAccountPermataChargeResponse', 'VirtualAccountBcaChargeResponse', 'VirtualAccountBniChargeResponse',
           'VirtualAccountMandiriChargeResponse', 'EpayBriChargeResponse',
           'build_charge_response', 'ApproveResponse',]

from veritranspay import mixins, helpers, payment_types


class ResponseBase(mixins.SerializableMixin):
    '''
    Base class for all responses from Veritrans.  The only two things
    we can be safely assured should be in every transaction are status_code
    and status_message.
    '''

    def __init__(self, status_code, status_message, *args, **kwargs):
        '''
        :param status_code: Transaction status code supplied by Veritrans.
        :type status_code: :py:class:`str`
        :param status_message: Human-readable status message.
        :type status_message: :py:class:`str`
        '''
        self.status_code = int(status_code)
        self.status_message = status_message

    def __repr__(self):
        return "<{klass}(code: {code}, message: {msg})>".format(
            klass=self.__class__.__name__,
            code=self.status_code,
            msg=self.status_message)


class ChargeResponseBase(ResponseBase):
    '''
    Encapsulates the response from Vertrans, returned after a
    :py:class:`veritranspay.request.ChargeRequest`.
    '''

    def __init__(self, *args, **kwargs):
        super(ChargeResponseBase, self).__init__(*args, **kwargs)
        self.transaction_id = kwargs.get('transaction_id', None)
        self.order_id = kwargs.get('order_id', None)
        self.payment_type = kwargs.get('payment_type', None)
        self.transaction_time = \
            helpers.parse_veritrans_datetime(
                kwargs.get('transaction_time', None))
        self.transaction_status = kwargs.get('transaction_status', None)
        self.fraud_status = kwargs.get('fraud_status', None)
        self.approval_code = kwargs.get('approval_code', None)
        self.gross_amount = \
            helpers.parse_veritrans_amount(
                kwargs.get('gross_amount', None))


class CreditCardChargeResponse(ChargeResponseBase):
    '''
    Response from Veritrans, returned after a
    :py:class:`veritrans.request.ChargeRequest` with a charge type of
    :py:class:`veritrans.payment_types.CreditCard`.
    '''

    def __init__(self, *args, **kwargs):
        super(CreditCardChargeResponse, self).__init__(*args, **kwargs)
        self.masked_card = kwargs.get('masked_card', None)
        self.bank = kwargs.get('bank', None)
        self.masked_card = kwargs.get('masked_card', None)
        self.saved_token_id = kwargs.get('saved_token_id')
        self.saved_token_id_expired_at = helpers.parse_veritrans_datetime(
            kwargs.get("saved_token_id_expired_at", None))


class IndomaretChargeResponse(ChargeResponseBase):
    """ CStore charge response when using payment_types.Indomaret

    http://docs.veritrans.co.id/en/vtdirect/integration_indomrt.html#response-transaction-indomrt
    """

    def __init__(self, *args, **kwargs):
        self.payment_code = kwargs.get('payment_code')
        super(IndomaretChargeResponse, self).__init__(*args, **kwargs)


class CimbsChargeResponse(ChargeResponseBase):
    """
    Cimb charge response when using payment_types is chimb clicks.

    http://api-docs.midtrans.com/#cimb-clicks
    """
    def __init__(self, *args, **kwargs):
        super(CimbsChargeResponse, self).__init__(*args, **kwargs)
        self.redirect_url = kwargs.get('redirect_url', None)


class MandiriChargeResponse(ChargeResponseBase):
    """
    Mandiri charge response when using payment_types is mandiri click pay.

    http://api-docs.midtrans.com/#mandiri-clickpay
    """
    def __init__(self, *args, **kwargs):
        super(MandiriChargeResponse, self).__init__(*args, **kwargs)
        self.masked_card = kwargs.get('masked_card', None)


class BCAKlikPayChargeResponse(ChargeResponseBase):
    """
    BCA charge response when using payment_types is bca_klikpay.

    http://api-docs.midtrans.com/#bca-klikpay
    """
    def __init__(self, *args, **kwargs):
        super(BCAKlikPayChargeResponse, self).__init__(*args, **kwargs)
        self.redirect_url = kwargs.get("redirect_url", None)


class KlikBCAChargeResponse(ChargeResponseBase):
    """
    BCA charge response when using payment_types is bca_klikbca.

    http://api-docs.midtrans.com/#klikbca
    """
    def __init__(self, *args, **kwargs):
        super(KlikBCAChargeResponse, self).__init__(*args, **kwargs)
        self.redirect_url = kwargs.get("redirect_url", None)


class VirtualAccountChargeResponse(ChargeResponseBase):
    # not implemented -- not documented
    def __init__(self, *args, **kwargs):
        super(VirtualAccountChargeResponse, self).__init__(*args, **kwargs)
        self.permata_va_number = kwargs.get('permata_va_number', None)


class VirtualAccountPermataChargeResponse(ChargeResponseBase):
    """
        Response from Veritrans, returned after a
        :py:class:`veritrans.request.ChargeRequest` with a charge type of
        :py:class:`veritrans.payment_types.VirtualAccountPermata`.

        https://api-docs.midtrans.com/#permata-virtual-account
    """

    def __init__(self, *args, **kwargs):
        super(VirtualAccountPermataChargeResponse, self).__init__(*args, **kwargs)
        self.bank = 'Permata'
        self.permata_va_number = kwargs.get('permata_va_number', None)


class VirtualAccountBcaChargeResponse(ChargeResponseBase):
    """
        Response from Veritrans, returned after a
        :py:class:`veritrans.request.ChargeRequest` with a charge type of
        :py:class:`veritrans.payment_types.VirtualAccountBca`.

        https://api-docs.midtrans.com/#bca-virtual-account
    """

    def __init__(self, *args, **kwargs):
        super(VirtualAccountBcaChargeResponse, self).__init__(*args, **kwargs)
        self.bank = 'Bca'
        self.va_numbers = kwargs.get('va_numbers', None)


class VirtualAccountBniChargeResponse(ChargeResponseBase):
    """
        Response from Veritrans, returned after a
        :py:class:`veritrans.request.ChargeRequest` with a charge type of
        :py:class:`veritrans.payment_types.VirtualAccountBni`.

        https://api-docs.midtrans.com/#bni-virtual-account
    """
    def __init__(self, *args, **kwargs):
        super(VirtualAccountBniChargeResponse, self).__init__(*args, **kwargs)
        self.bank = 'Bni'
        self.va_numbers = kwargs.get('va_numbers', None)


class VirtualAccountMandiriChargeResponse(ChargeResponseBase):
    """
        Response from Veritrans, returned after a
        :py:class:`veritrans.request.ChargeRequest` with a charge type of
        :py:class:`veritrans.payment_types.VirtualAccountMandiri`.

        https://api-docs.midtrans.com/#bni-virtual-account
    """
    def __init__(self, *args, **kwargs):
        super(VirtualAccountMandiriChargeResponse, self).__init__(*args, **kwargs)
        self.bank = 'Mandiri'
        self.bill_key = kwargs.get('bill_key', None)
        self.biller_code = kwargs.get('biller_code', None)


class EpayBriChargeResponse(ChargeResponseBase):
    """
        https://api-docs.midtrans.com/#epay-bri
    """
    def __init__(self, *args, **kwargs):
        super(EpayBriChargeResponse, self).__init__(*args, **kwargs)
        self.redirect_url = kwargs.get('redirect_url', None)


class GoPayChargeResponse(ChargeResponseBase):
    """
        https://api-docs.midtrans.com/#go-pay
    """
    def __init__(self, *args, **kwargs):
        super(GoPayChargeResponse, self).__init__(*args, **kwargs)
        self.actions = kwargs.get('actions', None)
        self.signature_key = kwargs.get('signature_key', None)
        self.channel_response_code = kwargs.get('channel_response_code', None)
        self.channel_response_message = kwargs.get('channel_response_message', None)
        self.currency = kwargs.get('currency', 'IDR')


def build_charge_response(request, *args, **kwargs):
    '''
    Builds a response appropriate for a given type of request.

    :param request: The request that was submitted to Veritrans.
        The charge_type is used to determine the appropriate
        response type to build.
    :type request: :py:class:`veritranspay.request.ChargeRequest`
    '''
    if isinstance(request, RegisterCardRequest):
        return RegisterCardResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.CreditCard):
        return CreditCardChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.Indomaret):
        return IndomaretChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.VirtualAccountPermata):
        return VirtualAccountPermataChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.VirtualAccountBca):
        return VirtualAccountBcaChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.VirtualAccountBni):
        return VirtualAccountBniChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.VirtualAccountMandiri):
        return VirtualAccountMandiriChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.BriEpay):
        return EpayBriChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.CimbClicks):
        return CimbsChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.MandiriClickpay):
        return MandiriChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.BCAKlikPay):
        return BCAKlikPayChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.KlikBCA):
        return KlikBCAChargeResponse(*args, **kwargs)
    elif isinstance(request.charge_type, payment_types.GoPay):
        return GoPayChargeResponse(*args, **kwargs)
    else:
        return ChargeResponseBase(*args, **kwargs)


class StatusResponse(ResponseBase):
    '''
    Encapsulates information about the status of a single charge.
    Returned from Veritrans after submitting a
    :py:class:`veritranspay.request.StatusRequest`
    '''

    def __init__(self, *args, **kwargs):
        super(StatusResponse, self).__init__(*args, **kwargs)
        self.transaction_id = kwargs.get('transaction_id', None)
        self.masked_card = kwargs.get('masked_card', None)
        self.order_id = kwargs.get('order_id', None)
        self.payment_type = kwargs.get('payment_type', None)
        self.transaction_time = \
            helpers.parse_veritrans_datetime(
                kwargs.get('transaction_time', None))
        self.transaction_status = kwargs.get('transaction_status', None)
        self.fraud_status = kwargs.get('fraud_status', None)
        self.approval_code = kwargs.get('approval_code', None)
        self.signature_key = kwargs.get('signature_key', None)
        self.bank = kwargs.get('bank', None)
        self.permata_va_number = kwargs.get('permata_va_number', None)
        self.va_number = kwargs.get('va_number', None)
        self.bill_key = kwargs.get('bill_key', None)
        self.biller_code = kwargs.get('biller_code', None)
        self.redirect_url = kwargs.get('redirect_url', None)
        self.gross_amount = \
            helpers.parse_veritrans_amount(
                kwargs.get('gross_amount', None))


class CancelResponse(ResponseBase):
    '''
    Data returned from Veritrans after submitting a
    :py:class:`veritranspay.request.CancelRequest`.
    '''

    def __init__(self, *args, **kwargs):
        super(CancelResponse, self).__init__(*args, **kwargs)
        self.transaction_id = kwargs.get('transaction_id', None)
        self.masked_card = kwargs.get('masked_card', None)
        self.order_id = kwargs.get('order_id', None)
        self.payment_type = kwargs.get('payment_type', None)
        self.transaction_time = \
            helpers.parse_veritrans_datetime(
                kwargs.get('transaction_time', None))
        self.transaction_status = kwargs.get('transaction_status', None)
        self.fraud_status = kwargs.get('fraud_status', None)
        self.approval_code = kwargs.get('approval_code', None)
        self.signature_key = kwargs.get('signature_key', None)
        self.bank = kwargs.get('bank', None)
        self.permata_va_number = kwargs.get('permata_va_number', None)
        self.va_number = kwargs.get('va_number', None)
        self.bill_key = kwargs.get('bill_key', None)
        self.biller_code = kwargs.get('biller_code', None)
        self.redirect_url = kwargs.get('redirect_url', None)
        self.gross_amount = \
            helpers.parse_veritrans_amount(
                kwargs.get('gross_amount', None))


class ApproveResponse(ResponseBase):
    '''
    Data returned from Veritrans after submitting a
    :py:class:`veritranspay.request.ApprovalRequest`
    '''

    def __init__(self, *args, **kwargs):
        super(ApproveResponse, self).__init__(*args, **kwargs)
        self.transaction_id = kwargs.get('transaction_id', None)
        self.masked_card = kwargs.get('masked_card', None)
        self.order_id = kwargs.get('order_id', None)
        self.payment_type = kwargs.get('payment_type', None)
        self.transaction_time = \
            helpers.parse_veritrans_datetime(
                kwargs.get('transaction_time', None))
        self.transaction_status = kwargs.get('transaction_status', None)
        self.fraud_status = kwargs.get('fraud_status', None)
        self.approval_code = kwargs.get('approval_code', None)
        self.signature_key = kwargs.get('signature_key', None)
        self.bank = kwargs.get('bank', None)
        self.permata_va_number = kwargs.get('permata_va_number', None)
        self.va_number = kwargs.get('va_number', None)
        self.bill_key = kwargs.get('bill_key', None)
        self.biller_code = kwargs.get('biller_code', None)
        self.redirect_url = kwargs.get('redirect_url', None)
        self.gross_amount = \
            helpers.parse_veritrans_amount(
                kwargs.get('gross_amount', None))


class RegisterCardResponse(mixins.SerializableMixin):
    '''
    Data returned from Veritrans after submitting a
    :py:class:`veritranspay.request.RegisterCardRequest`
    '''

    def __init__(self, *args, **kwargs):
        '''
        :param status_code: Transaction status code supplied by Veritrans.
        :type status_code: :py:class:`str`
        '''
        super(RegisterCardResponse, self).__init__()
        self.status_code = int(kwargs.get('status_code'))
        self.saved_token_id = kwargs.get('saved_token_id', None)
        self.transaction_id = kwargs.get('transaction_id', None)
        self.masked_card = kwargs.get('masked_card', None)

    def __repr__(self):
        return "<{klass}(code: {code})>".format(
            klass=self.__class__.__name__,
            code=self.status_code)
